"""A python libraty for working with macOS tags"""
from __future__ import annotations

import plistlib
import sys
from dataclasses import dataclass, field
from enum import Enum, unique
from pathlib import Path
from typing import Any, Dict, List, Optional, Sequence, Union

import mdfind  # type: ignore
import xattr  # type: ignore


def _test_os() -> None:
    """Raise an error if it's not MacOS."""
    if sys.platform != "darwin":
        raise RuntimeError(f"The library works only on macOS.")


_test_os()


__all__ = [
    "Color",
    "Tag",
    "find",
    "count",
    "get_all",
    "set_all",
    "remove_all",
    "add",
    "remove",
]


_XATTR_TAGS = "com.apple.metadata:_kMDItemUserTags"
_XATTR_FINDER_INFO = "com.apple.FinderInfo"
_MDFIND_TAGS_QUERY = "kMDItemUserTags=={}"
_ALL_TAGS_PLIST_PATH = f"{Path.home()}/Library/SyncedPreferences/com.apple.finder.plist"


@unique
class Color(Enum):
    GRAY = 1
    GREEN = 2
    PURPLE = 3
    BLUE = 4
    YELLOW = 5
    RED = 6
    ORANGE = 7

    def __str__(self) -> str:
        return str(self.value)


@dataclass(frozen=True)
class Tag:
    """Represents a single tag."""

    name: str
    color: Optional[Color] = field(default=None, compare=False)

    def __str__(self) -> str:
        if self.color:
            return f"{self.name}\n{self.color}"
        return self.name

    @classmethod
    def from_string(self, tag: str) -> Tag:
        """Create tag from string (like `"tag"` or `"tag\\n1"`)"""
        if "\n" in tag:
            name, color = tag.splitlines()
            return Tag(name, Color(int(color)))
        else:
            return Tag(tag)


AnyTag = Union[str, Tag]


def _create_tag(tag: AnyTag) -> Tag:
    return Tag.from_string(tag) if isinstance(tag, str) else tag


def _get_raw_tags(file: str) -> List[str]:
    # Like ["tag-one\n4", "tag-two\n6", tag-three"]
    plist = xattr.getxattr(file, _XATTR_TAGS)
    return plistlib.loads(plist)


def _remove_finder_info(file: str) -> None:
    # If you don't delete the com.apple.FinderInfo extended attributes,
    # Finder keeps showing the color labels for tags with colors.
    if _XATTR_FINDER_INFO in xattr.listxattr(file):
        xattr.removexattr(file, _XATTR_FINDER_INFO)


def find(tag: AnyTag, *, onlyin: Optional[str] = None) -> List[str]:
    """Find files by `tag`."""
    tag = _create_tag(tag)
    return mdfind.query(query=_MDFIND_TAGS_QUERY.format(tag.name), onlyin=onlyin)


def count(tag: AnyTag, *, onlyin: Optional[str] = None) -> int:
    """Output the total number of files by `tag`."""
    tag = _create_tag(tag)
    return mdfind.count(query=_MDFIND_TAGS_QUERY.format(tag.name), onlyin=onlyin)


def get_all(file: str) -> List[Tag]:
    """List the tags on the `file`."""
    return [Tag.from_string(tag) for tag in _get_raw_tags(file)]


def set_all(tags: Sequence[AnyTag], *, file: str) -> None:
    """Add `tags` to the `file` and remove the rest."""
    _remove_finder_info(file)
    plist = plistlib.dumps([str(tag) for tag in tags])  # type: ignore
    xattr.setxattr(file, _XATTR_TAGS, plist)


def remove_all(file: str) -> None:
    """Remove all tags from the `file`."""
    set_all([], file=file)


def add(tag: AnyTag, *, file: str) -> None:
    """Add `tag` to `file`."""
    tag = _create_tag(tag)
    tags = get_all(file)
    if tag not in tags:
        tags.append(tag)
        set_all(tags, file=file)


def remove(tag: AnyTag, *, file: str) -> None:
    """Remove `tag` from `file`."""
    tag = _create_tag(tag)
    tags = get_all(file)
    if tag in tags:
        tags.pop(tags.index(tag))
        set_all(tags, file=file)


def tags() -> List[Tag]:
    """Get all tags."""
    with open(_ALL_TAGS_PLIST_PATH, "rb") as fp:
        data: Dict["str", Any] = plistlib.load(fp)
        result = []
        for tag in data["values"]["FinderTagDict"]["value"]["FinderTags"]:
            color = None if tag.get("l") is None else Color(tag["l"])
            result.append(Tag(tag["n"], color))
        return result



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['macos_tags']
install_requires = \
['mdfind>=2019.4.13,<2020.0.0', 'xattr>=0.9.7,<0.10.0']

setup_kwargs = {
    'name': 'macos-tags',
    'version': '1.4.2',
    'description': 'Use tags to organize files on Mac from Python',
    'long_description': '# Use tags to organize files on Mac from Python\n\n![Release](https://github.com/dmkskn/macos-tags/workflows/Release/badge.svg)\n[![Checked with mypy](http://www.mypy-lang.org/static/mypy_badge.svg)](http://mypy-lang.org/)\n\n## Installation\n\n```bash\npip install macos-tags\n```\n\nWorks since Python 3.7.\n\n## Tutorial\n\nGet all tags:\n\n```python\n>>> import macos_tags\n\n\n>>> macos_tags.tags()\n[Tag(name=\'design\', color=None), ..., Tag(name=\'python\', color=<Color.GREEN: 2>]\n```\n\nGet files by tag name:\n\n```python\n>>> macos_tags.find("design")\n[\'/Users/home/apple.jpg\', \'/Users/home/WEB_POSTERS.png\']\n```\n\nCount files by tag name:\n\n```python\n>>> macos_tags.count("design")\n2\n```\n\nList the tags on the file:\n\n```python\n>>> path = "/path/to/file"\n\n>>> macos_tags.get_all(path)\n[Tag(name=\'design\', color=None), Tag(name=\'python\', color=<Color.GREEN: 2>]\n```\n\nAdd a tag to file:\n\n```python\n>>> macos_tags.add("design", file=path)\n```\n\n> When using `str` objects to define a tag, if a tag does not exist in the system, it will be added without a color label.\n\nAdd a new color tag by using `Tag` data class and `Color` enumeration:\n\n```python\n>>> from macos_tags import Tag, Color\n\n\n>>> tag = Tag(name="python", color=Color.GREEN)\n\n>>> macos_tags.add(tag, file=path)\n```\n\nAdd a new color tag using the `str` object, where the tag name and color number (from 1 to 7) are separated by the literal `\\n`:\n\n```python\n>>> tag = f"python\\n{Color.GREEN}"  # == "python\\n2"\n\n>>> macos_tags.add(tag, file=path)\n```\n\n> If the tag already exists in the system with a different color, the new color will be ignored.\n\nRemove tag from file:\n\n```python\n>>> macos_tags.remove(tag, file=path)\n```\n\nRemove all tags from a file at once:\n\n```python\n>>> macos_tags.remove_all(path)\n```\n\nChange all tags in the file:\n\n```python\n>>> macos_tags.get_all(path)\n[Tag(name=\'design\', color=None), Tag(name=\'python\', color=<Color.GREEN: 2>]\n\n>>> new_tags = [Tag("book"), Tag("programming", Color.BLUE)]\n\n>>> macos_tags.set_all(new_tags, file=path)\n\n>>> macos_tags.get_all(path)\n[Tag(name="book", color=None), Tag("programming", <Color.BLUE: 4>]\n```\n\n❤️',
    'author': 'Dima Koskin',
    'author_email': 'dmksknn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dmkskn/macos-tags',
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
